@@ -1,8 +1,8 @@
 /*
- * $Id: termin.gen.h 136 2006-09-22 20:06:05Z hubert@u.washington.edu $
+ * $Id: termin.gen.h 673 2007-08-16 22:25:10Z hubert@u.washington.edu $
  *
  * ========================================================================
- * Copyright 2006 University of Washington
+ * Copyright 2006-2007 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,6 +28,14 @@ int	process_config_input(UCS *);
 int	key_recorder(int);
 int	key_playback(int);
 int	recent_keystroke(int *, char *, size_t);
+int	init_tty_driver(struct pine *);
+void	end_tty_driver(struct pine *);
+int	PineRaw(int);
+UCS     read_char(int);
+void	init_keyboard(int);
+void	end_keyboard(int);
+int	pre_screen_config_opt_enter(char *, int, char *,
+				    ESCKEY_S *, HelpType, int *);
 #ifdef	_WINDOWS
 int	pcpine_oe_cursor(int, long);
 #endif
