@@ -23,12 +23,12 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 November 1989
- * Last Edited:	14 June 2007
+ * Last Edited:	8 October 2007
  */
 
 /* The Version */
 
-#define CCLIENTVERSION "2006j"
+#define CCLIENTVERSION "2006k"
 
 /* Build parameters */
 
@@ -366,7 +366,7 @@
 #define DR_CRLF (long) 0x200	/* driver internal form uses CRLF newlines */
 #define DR_NOSTICKY (long) 0x400/* driver does not support sticky UIDs */
 #define DR_RECYCLE (long) 0x800	/* driver does stream recycling */
-#define DR_XPOINT (long) 0x1000	/* needs to be checkpointed when recycling */
+#define DR_XPOINT (long) 0x1000	/* needs to be checkpointed */
 				/* driver has no real internal date */
 #define DR_NOINTDATE (long) 0x2000
 				/* driver does not announce new mail */
@@ -948,6 +948,8 @@ SEARCHPGM {			/* search program */
   STRINGLIST *to;		/* to recipients */
   unsigned long larger;		/* larger than this size */
   unsigned long smaller;	/* smaller than this size */
+  unsigned long older;		/* older than this interval */
+  unsigned long younger;	/* younger than this interval */
   unsigned short sentbefore;	/* sent before this date */
   unsigned short senton;	/* sent on this date */
   unsigned short sentsince;	/* sent since this date */
@@ -1615,6 +1617,7 @@ long mail_unsubscribe (MAILSTREAM *stream,char *mailbox);
 long mail_create (MAILSTREAM *stream,char *mailbox);
 long mail_delete (MAILSTREAM *stream,char *mailbox);
 long mail_rename (MAILSTREAM *stream,char *old,char *newname);
+char *mail_utf7_valid (char *mailbox);
 long mail_status (MAILSTREAM *stream,char *mbx,long flags);
 long mail_status_default (MAILSTREAM *stream,char *mbx,long flags);
 MAILSTREAM *mail_open (MAILSTREAM *stream,char *name,long options);
